<?php

namespace App\Controllers;

use App\Controllers\BaseController;
use CodeIgniter\HTTP\ResponseInterface;

// Models
use App\Models\PayrollGroupModel;
use App\Models\IncomeDeductionModel;
use App\Models\PayrollTypeModel;
use App\Models\EmployeePayrollInfoModel;
use App\Models\EmployeeModel;
use App\Models\EmpPayIncomeDeductionModel;
use App\Models\SettingsModel;
use App\Models\PayrollScheduleModel;
use App\Models\PayrollTransactionModel;
use App\Models\EmployeePayTransactionModel;
use App\Models\EmpPayTransIncomeDeductionModel;


// Entities
use App\Entities\PayrollGroup;
use App\Entities\IncomeDeduction;
use App\Entities\PayrollType;
use App\Entities\EmployeePayrollInfo;
use App\Entities\Employee;
use App\Entities\EmpPayIncomeDeduction;
use App\Entities\Settings;
use App\Entities\PayrollSchedule;
use App\Entities\PayrollTransaction;
use App\Entities\EmployeePayTransaction;
use App\Entities\EmpPayTransIncomeDeduction;

// Class Library
use App\ClassLib\MiscLib;
use App\ClassLib\PayrollComputation;

class PayrollController extends BaseController
{
    public function index()
    {
        return redirect()->to(base_url('/login'));
    }

    // Internally used functions

    private function computeEmployeePayroll($emppaytransid, $save = false, $instanceOfEmpPayTransModel = null, $instanceOfEmpPayTransInDedModel = null)
    {
        if($instanceOfEmpPayTransModel == null)
            $empPayTransModel = new EmployeePayTransactionModel();
        else
            $empPayTransModel = $instanceOfEmpPayTransModel;

        if($instanceOfEmpPayTransInDedModel == null)
            $empPayTransInDedModel = new EmpPayTransIncomeDeductionModel();
        else
            $empPayTransInDedModel = $instanceOfEmpPayTransInDedModel;
        
        $empPayTrans = $empPayTransModel->find($emppaytransid);

        // Compute basic pay first
        $empPayTrans->basic_pay = $empPayTrans->actual_work_days * $empPayTrans->basic_daily_pay;
        
        $empPayTrans->taxable_income = $empPayTransInDedModel->getTotalIncomeDeduction($emppaytransid, 1, 1) + $empPayTrans->basic_pay;
        $empPayTrans->nontaxable_income = $empPayTransInDedModel->getTotalIncomeDeduction($emppaytransid, 1, 0);
        $empPayTrans->taxable_deduction = $empPayTransInDedModel->getTotalIncomeDeduction($emppaytransid, 0, 1);
        $empPayTrans->nontaxable_deduction = $empPayTransInDedModel->getTotalIncomeDeduction($emppaytransid, 0, 0);

        // Compute gross first before getting taxable income to avoid misscalculation of net income
        $empPayTrans->gross_income = $empPayTrans->taxable_income + $empPayTrans->nontaxable_income;
        
        // Deduct SSS, Philhealth, Pagibig and Union Dues
        $empPayTrans->taxable_income -= $empPayTrans->nontaxable_deduction;

        // Compute Income Tax after getting taxable income
        $payCompute = new PayrollComputation();
        $empPayTrans->income_tax = $payCompute->computeIncomeTax($empPayTrans->taxable_income);

        // Deduction and net pay
        $empPayTrans->total_deduction = $empPayTrans->taxable_deduction + $empPayTrans->nontaxable_deduction + $empPayTrans->income_tax;
        $empPayTrans->net_pay = $empPayTrans->gross_income - $empPayTrans->total_deduction;

        if($save) $empPayTransModel->save($empPayTrans);
    }

    private function printRaw($label, $data)
    {
        echo $label.": ";
        print_r($empPayTrans->net_pay);
        echo "<br><br>";
    }


    // --end-- Internally used functions


    public function payrollGroup()
    {
        $payGroups = (new PayrollGroupModel())->findAll();

        $payGroupHTMLTable = new \CodeIgniter\View\Table();
        $payGroupHTMLTable->setTemplate(MiscLib::adminLTETableTemplate());

        if($payGroups == null)
            $data['tblPayGroup'] = '<p>No groups found.</p>';
        else
        {
            $payGroupHTMLTable->setHeading('ID', 'Group Code', 'Group Name', 'Action');

            foreach($payGroups as $group)
            {
                $iconView = '<a href="#" class="ml-3" data-toggle="tooltip" title="View Group Information"><i class="fas fa-eye "></i></a>';

                $payGroupHTMLTable->addRow($group->pay_group_id, $group->pay_group_code, $group->pay_group_name, "$iconView");
            }

            $data['tblPayGroup'] = $payGroupHTMLTable->generate();
        }

        return view('payroll/paygroupview', $data);
    }

    public function addPayrollGroup()
    {
        $payrollGroup = new PayrollGroup();
        $payrollGroupModel = new PayrollGroupModel();

        $rawData = $this->request->getPost();

        $payrollGroup->fill($rawData);
        $payrollGroupModel->save($payrollGroup);

        if($payrollGroupModel->getInsertID() == 0)
            return redirect()->back()->withInput()->with('error', 'Failed to add payroll group');
        else
            return redirect()->to('/payroll/paygroup')->with('message', 'Payroll Group Added');
    }

    public function incomeDeduction()
    {
        $incomeDeductions = (new IncomeDeductionModel())->findAll();

        $inDedHTMLTable = new \CodeIgniter\View\Table();
        $inDedHTMLTable->setTemplate(MiscLib::adminLTETableTemplate());

        if($incomeDeductions == null)
            $data['tblIncomeDeduction'] = '<p>No income and deduction found.</p>';
        else
        {
            $inDedHTMLTable->setHeading('ID', 'Payslip Display', 'COA Code', 'Deduction Name', 'Income', 'Taxable', 'Include in Gross', 'Action');

            foreach($incomeDeductions as $incomeDeduction)
            {
                $iconView = '<a href="#" class="ml-3" data-toggle="tooltip" title="View Information"><i class="fas fa-eye "></i></a>';

                $inDedHTMLTable->addRow($incomeDeduction->inded_id, $incomeDeduction->payslip_display, $incomeDeduction->coa_code, $incomeDeduction->income_deduction_name, ($incomeDeduction->is_income) ? 'Yes' : 'No', ($incomeDeduction->is_taxable) ? 'Yes' : 'No', ($incomeDeduction->include_in_gross) ? 'Yes' : 'No', $iconView);
            }

            $data['tblIncomeDeduction'] = $inDedHTMLTable->generate();
        }

        return view('payroll/incomedeductionview', $data);
    }

    public function addIncomeDeduction()
    {
        $incomeDeduction = new IncomeDeduction();
        $incomeDeductionModel = new IncomeDeductionModel();

        $rawData = $this->request->getPost();

        // Handle checkbox inputs
        $rawData['is_income'] = isset($rawData['is_income']) ? 1 : 0; // Set to 1 if checked, 0 if not
        $rawData['is_taxable'] = isset($rawData['is_taxable']) ? 1 : 0; // Same for taxable
        $rawData['include_in_gross'] = isset($rawData['include_in_gross']) ? 1 : 0; // Same for include in gross

        $incomeDeduction->fill($rawData);
        $incomeDeductionModel->save($incomeDeduction);

        if($incomeDeductionModel->getInsertID() == 0)
            return redirect()->back()->withInput()->with('error', 'Failed to add income or deduction');
        else
            return redirect()->to('/payroll/inded')->with('message', 'Income or Deduction Added');
    }

    public function payrollType()
    {
        $payrollTypes = (new PayrollTypeModel())->findAll();

        $payTypeHTMLTable = new \CodeIgniter\View\Table();
        $payTypeHTMLTable->setTemplate(MiscLib::adminLTETableTemplate());

        if($payrollTypes == null)
            $data['tblPayrollType'] = '<p>No payroll type found.</p>';
        else
        {
            $payTypeHTMLTable->setHeading('ID', 'Code', 'Name', 'Monthly', 'Semi-Monthly', 'Daily', 'Hourly', 'Action');

            foreach($payrollTypes as $payrollType)
            {
                $iconView = '<a href="#" class="ml-3" data-toggle="tooltip" title="View Information"><i class="fas fa-eye "></i></a>';

                $payTypeHTMLTable->addRow($payrollType->paytype_id, $payrollType->paytype_code, $payrollType->paytype_name, ($payrollType->is_monthly) ? 'Yes' : 'No', ($payrollType->is_semi_monthly) ? 'Yes' : 'No', ($payrollType->is_daily) ? 'Yes' : 'No', ($payrollType->is_hourly) ? 'Yes' : 'No', $iconView);
            }

            $data['tblPayrollType'] = $payTypeHTMLTable->generate();
        }

        return view('payroll/paytypeview', $data);
    }

    public function addPayrollType()
    {
        $payrollType = new PayrollType();
        $payrollTypeModel = new PayrollTypeModel();

        $rawData = $this->request->getPost();

        // Initialize all payroll type fields to 0
        $rawData['is_monthly'] = 0;
        $rawData['is_semi_monthly'] = 0;
        $rawData['is_daily'] = 0;
        $rawData['is_hourly'] = 0;

        // Handle radio button input
        if (isset($rawData['paytype_sched'])) {
            switch ($rawData['paytype_sched']) {
                case 'monthly':
                    $rawData['is_monthly'] = 1;
                    break;
                case 'semi_monthly':
                    $rawData['is_semi_monthly'] = 1;
                    break;
                case 'daily':
                    $rawData['is_daily'] = 1;
                    break;
                case 'hourly':
                    $rawData['is_hourly'] = 1;
                    break;
            }
        }

        $payrollType->fill($rawData);
        $payrollTypeModel->save($payrollType);

        if($payrollTypeModel->getInsertID() == 0)
            return redirect()->back()->withInput()->with('error', 'Failed to add payroll type');
        else
            return redirect()->to('/payroll/paytype')->with('message', 'Payroll Type Added');
    }

    public function employeePayrollInfo()
    {
        $empPayInfoModel = new EmployeePayrollInfoModel();
        $empPayInfos = $empPayInfoModel->getAllEmpPayInfoXEmpPayType();

        $data['employees'] = (new EmployeeModel())->findAll();
        $data['paytypes'] = (new PayrollTypeModel())->findAll();

        $empPayInfoHTMLTable = new \CodeIgniter\View\Table();
        $empPayInfoHTMLTable->setTemplate(MiscLib::adminLTETableTemplate());

        if($empPayInfos == null)
            $data['tblEmpPayInfo'] = '<p>No employee payroll type found.</p>';
        else
        {
            $empPayInfoHTMLTable->setHeading('ID', 'Payroll Type', 'Employee ID', 'Employee Name', 'Monthly', 'Semi-Monthly', 'Daily', 'Hourly', 'Action');

            foreach($empPayInfos as $empPayInfo)
            {
                $iconView = '<a href="#" class="ml-3" data-toggle="tooltip" title="View Information"><i class="fas fa-eye "></i></a>';

                $empPayInfoHTMLTable->addRow($empPayInfo->emppay_id, $empPayInfo->paytype_name, $empPayInfo->company_issued_id, $empPayInfo->last_name . ', ' . $empPayInfo->first_name, $empPayInfo->basic_monthly_pay, $empPayInfo->basic_semi_monthly_pay, $empPayInfo->basic_daily_pay, $empPayInfo->basic_hourly_pay, $iconView);
            }

            $data['tblEmpPayInfo'] = $empPayInfoHTMLTable->generate();
        }

        return view('payroll/empinfoview', $data);
    }

    public function addEmployeePayrollInfo()
    {
        $empPayInfo = new EmployeePayrollInfo();
        $empPayInfoModel = new EmployeePayrollInfoModel();

        $rawData = $this->request->getPost();

        $rawData['is_ATM'] = isset($rawData['is_ATM']) ? 1 : 0;
        $rawData['has_cola'] = isset($rawData['has_cola']) ? 1 : 0;
        $rawData['has_philhealth'] = isset($rawData['has_philhealth']) ? 1 : 0;
        $rawData['has_hdmf'] = isset($rawData['has_hdmf']) ? 1 : 0;
        $rawData['has_sss'] = isset($rawData['has_sss']) ? 1 : 0;
        $rawData['has_gsis'] = isset($rawData['has_gsis']) ? 1 : 0;

        $empPayInfo->fill($rawData);
        $empPayInfoModel->save($empPayInfo);

        if($empPayInfoModel->getInsertID() == 0)
            return redirect()->back()->withInput()->with('error', 'Failed to add employee payroll type');
        else
            return redirect()->to('/payroll/emppayinfo')->with('message', 'Employee Payroll Type Added');
    }

    public function employeeCompensationBenefits()
    {
        $empPayInfoModel = new EmployeePayrollInfoModel();
        $incomeDeductionModel = new IncomeDeductionModel();
        $empPayInDedModel = new EmpPayIncomeDeductionModel();

        $data['paySchedules'] = (new PayrollScheduleModel())->findAll();
        $data['incomeList'] = $incomeDeductionModel->where("is_income", 1)->findAll();
        $data['deductionList'] = $incomeDeductionModel->where("is_income", 0)->findAll();
        $data['empPayInfos'] = $empPayInfoModel->getAllEmpPayInfoXEmpPayType();
        $data['empLoaded'] = false;

        if($this->request->getGet('empid') != null) 
        {
            $settingsModel = new SettingsModel();

            $iconView = '<a href="#" class="ml-3" data-toggle="tooltip" title="View Employee Information" ><i class="fas fa-eye "></i></a>';
            $iconEdit = '<a href="#" class="ml-3" data-toggle="tooltip" title="Edit Employee Information" ><i class="fas fa-edit "></i></a>';
            $iconDelete = '<a href="#" class="ml-3" data-toggle="tooltip" title="Delete Employee Information"><i class="fas fa-trash "></i></a>';

            $data['empLoaded'] = true;
            $data['selectedEmployee'] = $empPayInfoModel->getEmpPayInfoXEmpPayTypeByEmpID($this->request->getGet('empid'));
            $data['empPayIncomeList'] = $empPayInDedModel->getEmpPayInDedByEmpPayIdSchedIdIsIncome($data['selectedEmployee']->emppay_id, $this->request->getGet('payschedid'), true);
            $data['empPayDeductionList'] = $empPayInDedModel->getEmpPayInDedByEmpPayIdSchedIdIsIncome($data['selectedEmployee']->emppay_id, $this->request->getGet('payschedid'),false);
        }

        return view('payroll/compensationbenefitsview', $data);
    }

    public function addEmployeeCompensationBenefits()
    {
        $empPayInDed = new EmpPayIncomeDeduction();
        $empPayInDedModel = new EmpPayIncomeDeductionModel();

        $rawData = $this->request->getPost();

        // Initialize all payroll type fields to 0
        $rawData['is_fixed_amt'] = 0;
        $rawData['is_percent_amt'] = 0;

        // Handle radio button input
        if (isset($rawData['amount_type'])) {
            switch ($rawData['amount_type']) {
                case 'fixed':
                    $rawData['is_fixed_amt'] = 1;
                    break;
                case 'perc':
                    $rawData['is_percent_amt'] = 1;
                    break;
            }
        }

        $rawData['is_override'] = isset($rawData['is_override']) ? 1 : 0;

        $empPayInDed->fill($rawData);

        if($empPayInDedModel->save($empPayInDed))
        {
            if(isset($rawData['emppayinded_id']))
                return redirect()->to('/payroll/compben?payschedid='.$this->request->getPost('payschedule_id').'&empid='.$this->request->getPost('emp_id'))->with('message', 'Employee Compensation Benefits edited');
            else
                return redirect()->to('/payroll/compben?payschedid='.$this->request->getPost('payschedule_id').'&empid='.$this->request->getPost('emp_id'))->with('message', 'Employee Compensation Benefits Added');
        }
        else
            return redirect()->back()->withInput()->with('error', 'Failed to add employee compensation benefits');
    }

    public function deleteEmployeeCompensationBenefits()
    {
        $empPayInDedModel = new EmpPayIncomeDeductionModel();

        if($empPayInDedModel->delete($this->request->getPost('emppayinded_id')))
            return redirect()->to('/payroll/compben?payschedid='.$this->request->getPost('payschedule_id').'&empid='.$this->request->getPost('emp_id'))->with('message', 'Employee Compensation Benefits deleted');
        else
            return redirect()->back()->with('error', 'Failed to delete employee compensation benefits');
    }

    public function payrollSettings()
    {
        $incomeDeductionModel = new IncomeDeductionModel();
        $settingsModel = new SettingsModel();
        $settings = new Settings();

        if($this->request->is('post'))
        {
            $settings->fill($this->request->getPost());
            $settingsModel->save($settings);
        }

        $data['indedList'] = $incomeDeductionModel->findAll();
        $data['basicSalVal'] = $settingsModel->where('key', 'BasicSalary')->first();
        $data['COLAVal'] = $settingsModel->where('key', 'COLA')->first();
        $data['PhilhealthVal'] = $settingsModel->where('key', 'Philhealth')->first();
        $data['HDMFVal'] = $settingsModel->where('key', 'HDMF')->first();
        $data['SSSVal'] = $settingsModel->where('key', 'SSS')->first();
        $data['GSISVal'] = $settingsModel->where('key', 'GSIS')->first();

        return view('payroll/paysettingsview', $data);
    }

    public function payrollTransactions()
    {
        $data['paytypes'] = (new PayrollTypeModel())->findAll();
        $data['paySchedules'] = (new PayrollScheduleModel())->findAll();
        
        $payTrans = (new PayrollTransactionModel())->orderBy('created_at', 'DESC')->findAll();

        $payTransHTMLTable = new \CodeIgniter\View\Table();
        $payTransHTMLTable->setTemplate(MiscLib::adminLTETableTemplate());

        if($payTrans == null)
            $data['tblPayTrans'] = '<p>No transactions found.</p>';
        else
        {
            $payTransHTMLTable->setHeading('ID', 'From', 'To', 'No. of Days', 'Status', 'Remarks', 'Action');

            foreach($payTrans as $trans)
            {
                $iconEdit = '<a href="/payroll/emppaytrans/'.$trans->paytrans_id.'" class="ml-3" data-toggle="tooltip" title="Edit Payroll Transaction"><i class="fas fa-edit"></i></a>';

                $payTransHTMLTable->addRow($trans->paytrans_id, $trans->payroll_from, $trans->payroll_to, $trans->no_of_days, $trans->is_open ? 'Open' : ' Closed', $trans->remarks, $iconEdit);
            }

            $data['tblPayTrans'] = $payTransHTMLTable->generate();
        }

        return view('payroll/paytransactionview', $data);
    }

    public function addPayrollTransactions()
    {
        $payTransModel = new PayrollTransactionModel();
        $payTrans = new PayrollTransaction();

        $rawData = $this->request->getPost();

        $payTrans->fill($rawData);
        
        if($payTransModel->save($payTrans))
            return redirect()->to('/payroll/paytrans')->with('message', 'Payroll Transaction Added');
        else
            return redirect()->back()->withInput()->with('error', 'Failed to add payroll transaction');
    }

    public function employeePayrollTransactions($paytransid)
    {
        $data['paygroupid'] = $this->request->getGet('grpid');
        $data['showInitBtn'] = false;
        
        $payTrans = (new PayrollTransactionModel())->where('paytrans_id', $paytransid)->first();
        $data['paytransid'] = $paytransid;
        $data['initURL'] = $paytransid.'/'.$data['paygroupid'].'/'.$payTrans->paytype_id;

        $data['paygroups'] = (new PayrollGroupModel())->findAll();

        $empPayTrans = (new EmployeePayTransactionModel())->getEmpPayTransByPayGroupId($data['paygroupid']);

        $empPayTransHTMLTable = new \CodeIgniter\View\Table();
        $empPayTransHTMLTable->setTemplate(MiscLib::adminLTETableTemplate());
        
        if($empPayTrans == null)
        {
            $empPayTrans = (new EmployeePayrollInfoModel())->getEmpPayInfoXEmpPayTypeByPayGrpId($data['paygroupid'], $payTrans->paytype_id);
            $data['showInitBtn'] = true;

            $empPayTransHTMLTable->setHeading('ID', 'Company ID', 'Name', 'Branch', 'Daily Basic');

            foreach($empPayTrans as $trans)
            {

                $empPayTransHTMLTable->addRow($trans->employee_id, $trans->company_issued_id, $trans->last_name . ', ' . $trans->first_name, $trans->branch_code, $trans->basic_monthly_pay);
            }

            $data['tblEmpPayTrans'] = $empPayTransHTMLTable->generate();
        }
        else
        {
            $empPayTransHTMLTable->setHeading('ID', 'Name', 'Branch', 'Basic Salary', 'Days Work', 'Gross', 'Deduction', 'Net', 'Action');

            /* `emppaytrans_id`, `paytrans_id`, `company_id`, `branch_code`, `dept_id`, `job_title_id`, 
            `pay_group_id`, `emp_status_id`, `employee_id`, `company_issued_id`, `last_name`, `first_name`, 
            `middle_name`, `suffix`, `email_address`, `is_ATM`, `savings_account`, `basic_monthly_pay`, `basic_daily_pay`, 
            `basic_hourly_pay`, `has_cola`, `has_philhealth`, `has_hdmf`, `has_sss`, `has_gsis`, `actual_work_days`, `basic_pay`, 
            `gross_income`, `taxable_income`, `nontaxable_income`, `income_tax`, `total_deduction`, `taxable_deduction`, `nontaxable_deduction`, 
            `net_pay`, `created_at`, `created_by`, `updated_at`, `updated_by`, `deleted_at`, `paytrans_id`, `paytype_id`, `payschedule_id`, `payroll_from`, 
            `payroll_to`, `no_of_days`, `total_emp`, `total_gross`, `remarks`, `is_open`, `created_at`, `created_by`, `updated_at`, 
            `updated_by`, `deleted_at`, `branch_code`, `company_id`, `branch_name`, `address`, `contact_number`, `email_address`,
             `created_at`, `created_by`, `updated_at`, `updated_by`, `deleted_at`, `dept_id`, `company_id`, `parent_dept_id`, 
             `department_code`, `department_name`, `created_at`, `created_by`, `updated_at`, `updated_by`, `deleted_at`, `job_title_id`, 
             `job_title_name`, `created_at`, `created_by`, `updated_at`, `updated_by`, `deleted_at`, `pay_group_id`, `pay_group_code`, 
             `pay_group_name`, `created_at`, `created_by`, `updated_at`, `updated_by`, `deleted_at`, `emp_status_id`, `status_name`, 
             `created_at`, `created_by`, `updated_at`, `updated_by`, `deleted_at`, `employee_id`, `company_id`, `branch_code`, `dept_id`, 
             `job_title_id`, `emp_status_id`, `pay_group_id`, `company_issued_id`, `last_name`, `first_name`, `middle_name`, `suffix`, `email_address`, 
             `contact_number`, `created_at`, `created_by`, `updated_at`, `updated_by`, `deleted_at` 
             
             
             [emppaytrans_id] => 30 [paytrans_id] => 1 [company_id] => 1 [branch_code] => KWSMB [dept_id] => 1 [job_title_id] => 1 [pay_group_id] => 1 
             [emp_status_id] => 1 [employee_id] => 1 [company_issued_id] => 1900 [last_name] => bel [first_name] => paul [middle_name] => b [suffix] => 
             [email_address] => user1@yahoo.com [is_ATM] => 1 [savings_account] => 0000 [basic_monthly_pay] => 20000.0000 [basic_daily_pay] => 766.7732 
             [basic_hourly_pay] => 95.8466 [has_cola] => 0 [has_philhealth] => 1 [has_hdmf] => 1 [has_sss] => 1 [has_gsis] => 0 [actual_work_days] => 8.0000 
             [basic_pay] => 6134.1856 [gross_income] => 12268.3712 [taxable_income] => 12268.3712 [nontaxable_income] => 0.0000 [income_tax] => 370.2742 
             [total_deduction] => 400.0000 [taxable_deduction] => 0.0000 [nontaxable_deduction] => 400.0000 [net_pay] => 0.0000 
             [created_at] => 2024-09-06 06:36:22 [created_by] => 11-E128-70 [updated_at] => 2024-10-05 04:21:57 [updated_by] => 11-E128-70 [deleted_at] => 
             [paytype_id] => 1 [payschedule_id] => 1 [payroll_from] => 2024-10-03 [payroll_to] => 2024-10-10 [no_of_days] => 8 [total_emp] => 0 [total_gross] => 0.0000 
             [remarks] => [is_open] => 1 [branch_name] => Karat World SM City Bacolod [address] => Bacolod City [contact_number] => 000 [parent_dept_id] => 0 
             [department_code] => HR [department_name] => Human Resources [job_title_name] => HR Personnel [pay_group_code] => VIS [pay_group_name] => Visayas Group 
             [status_name] => AWOL

             
             [emppaytransinded_id] => 16 [emppaytrans_id] => 30 [inded_id] => 1 [payslip_display] => SSS Contribution [inded_name] => SSS 
             [coa_code] => [is_income] => 0 [is_taxable] => 0 [include_in_gross] => 0 [is_fixed_amt] => 1 [is_percent_amt] => 0 
             [amount] => 300.0000 [base_amount] => 300.0000 [is_override] => 0 [created_at] => 2024-09-13 09:34:54 [created_by] => 11-E128-70 
             [updated_at] => [updated_by] => [deleted_at] => */

            foreach($empPayTrans as $trans)
            {
                //$iconView = '<a href="#" class="ml-3" data-toggle="tooltip" title="View Information" onclick="showEmpPayTransDetails('.$trans->emppaytrans_id.')"><i class="fas fa-eye "></i></a>';
                $iconEdit = '<a href="#" class="ml-3" data-toggle="tooltip" title="Edit Information" onclick="showEmpPayTransDetails('.$trans->emppaytrans_id.')"><i class="fas fa-edit "></i></a>';

                $empPayTransHTMLTable->addRow($trans->employee_id, $trans->last_name . ', ' . $trans->first_name, $trans->branch_code, $trans->basic_pay, $trans->actual_work_days, $trans->gross_income, $trans->total_deduction, $trans->net_pay, $iconEdit);

                // Get Emp Trans In Ded
                $empPayTransInDedModel = new EmpPayTransIncomeDeductionModel();
                $data['emppaytrans'][] = ["empPayTrans"=>$trans, 
                                          "empPayTransInDed"=>$empPayTransInDedModel->getEmpPayTransInDedxInDedByEmpTransId($trans->emppaytrans_id)];
            }

            $data['tblEmpPayTrans'] = $empPayTransHTMLTable->generate();
        }

        return view('payroll/emppaytransactionview', $data);
    }

    public function empPayTransInitializePayroll($paytransid, $paygroupid, $transtypid)
    {
        $empPayInfos = (new EmployeePayrollInfoModel())->getEmpPayInfoXEmpPayTypeByPayGrpId($paygroupid, $transtypid);
        $payTrans = (new PayrollTransactionModel())->where('paytrans_id', $paytransid)->first();
        $empPayTransactionModel = new EmployeePayTransactionModel();
        
        foreach($empPayInfos as $empPayInfo)
        {
            $empPayTransaction = new EmployeePayTransaction();
           
            $empPayTransaction->fill(
                ['paytrans_id' => $paytransid,
                'company_id' => $empPayInfo->company_id,
                'branch_code' => $empPayInfo->branch_code,
                'dept_id' => $empPayInfo->dept_id,
                'job_title_id' => $empPayInfo->job_title_id,
                'pay_group_id' => $empPayInfo->pay_group_id,
                'emp_status_id' => $empPayInfo->emp_status_id,
                'employee_id' => $empPayInfo->employee_id,
                'company_issued_id' => $empPayInfo->company_issued_id,
                'last_name' => $empPayInfo->last_name,
                'first_name' => $empPayInfo->first_name,
                'middle_name' => $empPayInfo->middle_name,
                'suffix' => $empPayInfo->suffix,
                'email_address' => $empPayInfo->email_address,
                'is_ATM' => $empPayInfo->is_ATM,
                'savings_account' => $empPayInfo->savings_account,
                'basic_monthly_pay' => $empPayInfo->basic_monthly_pay,
                'basic_daily_pay' => $empPayInfo->basic_daily_pay,
                'basic_hourly_pay' => $empPayInfo->basic_hourly_pay,
                'has_cola' => $empPayInfo->has_cola,
                'has_philhealth' => $empPayInfo->has_philhealth,
                'has_hdmf' => $empPayInfo->has_hdmf,
                'has_sss' => $empPayInfo->has_sss,
                'has_gsis' => $empPayInfo->has_gsis,
                'actual_work_days' => $payTrans->no_of_days,
                'basic_pay' => $payTrans->no_of_days * $empPayInfo->basic_daily_pay,
                'gross_income' => 0,
                'taxable_income' => 0,
                'nontaxable_income' => 0,
                'income_tax' => 0,
                'total_deduction' => 0,
                'taxable_deduction' => 0,
                'nontaxable_deduction' => 0,
                'net_pay' => 0]
            );
            
            $empPayTransactionModel->save($empPayTransaction);
            $empPayTransaction = $empPayTransactionModel->find($empPayTransactionModel->getInsertID());

            $empPayInDeds = (new EmpPayIncomeDeductionModel())->getEmpPayInDedByEmpPayIdSchedId($empPayInfo->emppay_id, $payTrans->payschedule_id);

            foreach($empPayInDeds as $empPayInDed)
            {
                $empPayTransInDed = new EmpPayTransIncomeDeduction();
                $empPayTransInDedModel = new EmpPayTransIncomeDeductionModel();

                $amount = $empPayInDed->is_fixed_amt ? $empPayInDed->amount : ($empPayInDed->amount / 100) * $empPayTransaction->basic_pay;

                $empPayTransInDed->fill(
                    ['emppaytrans_id' => $empPayTransaction->emppaytrans_id,
                    'inded_id' => $empPayInDed->inded_id,
                    'payslip_display' => $empPayInDed->payslip_display,
                    'inded_name' => $empPayInDed->inded_name,
                    'coa_code' => $empPayInDed->coa_code,
                    'is_income' => $empPayInDed->is_income,
                    'is_taxable' => $empPayInDed->is_taxable,
                    'include_in_gross' => $empPayInDed->include_in_gross,
                    'is_fixed_amt' => $empPayInDed->is_fixed_amt,
                    'is_percent_amt' => $empPayInDed->is_percent_amt,
                    'amount' => $amount,
                    'base_amount' => $empPayInDed->amount,
                    'is_override' => $empPayInDed->is_override]
                );

                $empPayTransInDedModel->save($empPayTransInDed);

                // Updating EmpPayTransaction
                
                if($empPayInDed->is_income)
                {
                    $empPayTransaction->gross_income += $amount;
                    $empPayTransaction->{$empPayInDed->is_taxable ? 'taxable_income' : 'nontaxable_income'} += $amount;
                }
                else
                {
                    $empPayTransaction->total_deduction += $amount;
                    $empPayTransaction->{$empPayInDed->is_taxable ? 'taxable_deduction' : 'nontaxable_deduction'} += $amount;
                }

                // Less nontaxable deduction to taxable income
                $empPayTransaction->taxable_income -= $empPayTransaction->nontaxable_income;

                // Compute Income Tax
                $payCompute = new PayrollComputation();
                $empPayTransaction->income_tax = $payCompute->computeIncomeTax($empPayTransaction->taxable_income);
            }

            $empPayTransaction->gross_income += $empPayTransaction->basic_pay;
            $empPayTransaction->taxable_income += $empPayTransaction->basic_pay;
            $empPayTransaction->total_deduction += $empPayTransaction->income_tax;
            $empPayTransaction->net_pay = $empPayTransaction->gross_income - $empPayTransaction->total_deduction;

            $empPayTransactionModel->save($empPayTransaction);
        }

        return redirect()->back()->withInput()->with('message', 'Payroll processed. Please verify the entries.');
    }

    public function employeePayrollTransactionsEditDaysWorked()
    {
        $empPayTransModel = new EmployeePayTransactionModel();

        $rawData = $this->request->getPost();

        $empPayTransModel->changeActualDaysWorked($rawData['emppaytrans_id'], $rawData['actual_work_days']);
        $this->computeEmployeePayroll($rawData['emppaytrans_id'], true, $empPayTransModel);

        return redirect()->back()->withInput()->with('message', 'Days worked updated.');
    }
}
